#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
**Retrieves and displays meteorological radiosonde data**

(rudimentary support for soundings from the University of Wyoming
http://weather.uwyo.edu/cgi-bin/sounding )

It can both be used as a library and as a stand alone program

|     ./adrasolib.py [statnr [fromtime] ]
| eg. ./adrasolib.py 11010 03

The radiosonde from Linz-Hoersching (11010) with a start time between
03UTC and 03+6 = 09UTC is retrieved and plot to a file in
the directory ~/rasograph/ .


.. note:: This module makes use of the **numpy** and **matplotlib** lybraries.

   NumPy is the fundamental package for scientific computing with Python. See http://www.numpy.org/ and
   http://sourceforge.net/projects/numpy/

   Matplotlib is the most prominent python scientific plotting library. See
   https://matplotlib.org/

   Make sure you have them installed.

The html-stripping script of http://www.aaronsw.com/2002/html2text/
is renamed to adhmtl2txt and is part of the distribution.


This program is available under the license GNU GPL 3.0.
http://www.gnu.org/licenses/gpl-3.0

| (c) D.Thaler, 2013-2019
| E-mail: info@foehnwall.at
"""

import StringIO
import datetime
import os
import sys
import tempfile
import urllib2

import adhtml2text as aht
import admeteopy as am
import matplotlib.pyplot as pl
import numpy as np
import os.path as op

# _VERSION = '0.1.1-2013-04-10' #: Module version
_VERSION = '0.3.0-2019-06-23'  # : Module version
WYOMINGRASOREGIONS = {'europe', 'mideast', 'naconf', 'samer', 'np', 'ant', \
            'africa', 'seasia', 'pac', 'nz'}  # : List of regions
__version__ = _VERSION


def getrasowyoming(statnr, region, year, month, fromddhh, toddhh):
    """Get Raso-data from the University of Wyoming
    http://weather.uwyo.edu/cgi-bin/sounding and convert them into a list of
    text data that can be processed further. with help of modul adhtml2text
    that is derived from http://www.aaronsw.com/2002/html2text/

        | statnr   ..  stationnumber as a string
        | region   ..  an element of WYOMINGRASOREGIONS
        | year     ..  as string
        | month    ..  as string
        | fromddhh ..  like '3100'
        | toddhh   ..  like '3112'
    """
    url0 = "http://weather.uwyo.edu/cgi-bin/sounding?"
    if not (region in WYOMINGRASOREGIONS):
        print "Error: No valid Region for Wyoming-Rasos:", region
        print "Region must be a member of "
        for x in WYOMINGRASOREGIONS:
            print x,
            print
            sys.exit(99)
    url1 = "region=" + region + "&TYPE=TEXT%3ALIST&"
    url2 = "YEAR=%04d&MONTH=%02d&FROM=%04d&TO=%04d&STNM=%s" % \
                (year, month, fromddhh, toddhh, statnr)
    url = url0 + url1 + url2
    response = urllib2.urlopen(url)
    headers = response.info()
    data_html = response.read()
    datalength = len(data_html)
    data_txt = aht.html2text(data_html)
    if datalength < 1500:
        return None
    else:
        return data_txt


def parserasodata(rasodata):
    """Converts html-stripped raso-text data in the University of
    Wyoming format from input-list "rasodata" to
    numpy 2-D data arrays that can be processed further.
    """
    # Still a bit confusing code due to "historical" reasons
    buf = StringIO.StringIO(rasodata)
    rawdata = []
    for ll in buf:
        rawdata.append(ll.strip())
    data = []
    for k in range(len(rawdata)):
        if rawdata[k].startswith("## "):
            specline = rawdata[k].split()
            statnr = specline[1]
            statname = specline[-7]
            yyyy = specline[-1]
            mm = specline[-2]
            dd = specline[-3]
            hh = specline[-4]
            datestr = yyyy + '-' + mm + '-' + dd + '-' + hh
        if rawdata[k].startswith("-------------"):
            k = k + 4
            while not rawdata[k].startswith("###"):
                data.append(rawdata[k])
                k = k + 1
            break
    raso = []
    for k in range(len(data) - 2):
        raso.append(data[k])
    xraso = []
    for k in range(len(raso)):
        xs = raso[k].split()
        x = []
        for l in xs:
            x.append(float(l))
        if len(x) == 11:
            xraso.append(x)
    rraso = np.asarray(xraso)
    pres = rraso[:, 0]
    hght = rraso[:, 1]
    temp = rraso[:, 2]
    dwpt = rraso[:, 3]
    relh = rraso[:, 4]
    mixr = rraso[:, 5]
    dcrt = rraso[:, 6]
    sknt = rraso[:, 7]
    thta = rraso[:, 8]
    thte = rraso[:, 9]
    thtv = rraso[:, 10]
    return statnr, statname, datestr, \
        pres, hght, temp, dwpt, relh, mixr, dcrt, sknt, thta, thte, thtv


def getallrasodata(fpath):
    flist = os.listdir(fpath)


def savetext2file(text, pathname):
    f = STDIO
    return False


def plot_raso_simple(graphpath, rasodata):
    """Plots a simple temperature and dew point difference plot versus
    pressur and safes the graphic to "graphpath" from the parsed
    Wyoming-Raso-Data in "rasodata" """
    (nummer, name, datum, pres, hght, temp, dwpt, relh, mixr, dcrt, sknt, thta, thte, thtv) = rasodata
    fn = nummer + "-" + datum + ".png"
    fname = os.path.join(graphpath, fn)
    (nummer, name, datum, pres, hght, temp, dwpt, relh, mixr, dcrt, sknt, thta, thte, thtv) = \
    rasodata
    pl.plot(temp, pres, color="blue", lw=2)
    pl.plot((dwpt - temp) - 20, pres, color='green', lw=1, linestyle='--')
    pl.grid()
    pl.gca().invert_yaxis()
    pl.ylim(1050, 100)
    pl.xlim(-70.0, +20.0)
    pl.xlabel("Celsius")
    pl.ylabel("hPa")
    pl.minorticks_on()
    pl.title(nummer + " " + name + " " + datum)
    pl.savefig(fname)
    pl.close('all')
    return fname


def _test():
    GRAPHPATH = os.path.expanduser("~/rasograph")
    if not op.exists(GRAPHPATH):
        os.makedirs(GRAPHPATH)
    try:
        station_ = sys.argv[1]
    except IndexError:
        station_ = '11035'
    try:
        fh = max(int(sys.argv[2]), 0)
    except IndexError:
        fh = 0
    th = min(fh + 6, 23)
    region_ = 'europe'
    heute = datetime.date.today()
    yy = heute.year; mm = heute.month; dd = heute.day
    from_dh = dd * 100 + fh; til_dh = dd * 100 + th
    data_txt = getrasowyoming(station_, region_, yy, mm, from_dh, til_dh)
    if not (data_txt == None):
        data_raso = parserasodata(data_txt)
    else:
        print "No radio sounding ", station_, " of ", heute, " between ", \
            fh, " and ", th, " UTC available"
        sys.exit(10)
    fn = plot_raso_simple(GRAPHPATH, data_raso)
    print "Radio sounding", station_, "of", heute, "between", \
            fh, "and", th, "UTC available,\ngraph saved in", fn
    return


if __name__ == "__main__":
    _test()
